/**
 * Идея: посторим триангулированную выпуклую оболочку, на запрос будем находить
 * нужный треугольник, выбирать в нём какую-то точку и переходить к одной из
 * трёх частей.
 * Время: n log n + qn
 * Запросы: q (n + log n)
 * Баллы: 50-60
 */

#include <bits/stdc++.h>

using namespace std;
vector<int> x, y;
long long area(int a, int b, int c)
{
	return 1LL * (x[b] - x[a]) * (y[c] - y[a]) - 1LL * (x[c] - x[a]) * (y[b] - y[a]);
}
bool cmp(int x, int y)
{
	return area(0, x, y) > 0;
}
vector<int> convexHull(vector<int> a)
{
	int minPos = 0;
	for (int i = 0; i < (int)a.size(); i++)
	{
		if (make_pair(x[a[i]], y[a[i]]) < make_pair(x[a[minPos]], y[a[minPos]]))
		{
			minPos = i;
		}
	}
	if (minPos != 0)
	{
		swap(a[minPos], a[0]);
	}
	pair<int, int> mn = make_pair(x[a[0]], y[a[0]]);
	for (int i = 0; i < (int)a.size(); i++)
	{
		x[a[i]] -= mn.first;
		y[a[i]] -= mn.second;
	}
	sort(a.begin() + 1, a.end(), cmp);
	vector<int> st;
	for (int i = 0; i < (int)a.size(); i++)
	{
		while ((int)st.size() >= 2 && area(st[(int)st.size() - 2], st[(int)st.size() - 1], a[i]) <= 0) st.pop_back();
		st.push_back(a[i]);
	}
	for (int i = 0; i < (int)a.size(); i++)
	{
		x[a[i]] += mn.first;
		y[a[i]] += mn.second;
	}
	return st;
}
bool ask(int a, int b, int c)
{
	cout << "? 3 " << a << " " << b << " " << c << endl;
	string res;
	cin >> res;
	return res == "Yes";
}
bool inside(int a, int b, int c, int x)
{
	return abs(area(a, b, c)) == abs(area(a, b, x)) + abs(area(b, c, x)) + abs(area(a, c, x));
}
int main()
{
	int n;
	cin >> n;
	x.resize(n + 1);
	y.resize(n + 1);
	for (int i = 1; i <= n; i++)
	{
		cin >> x[i] >> y[i];
	}
	int q;
	cin >> q;
	for (int qq = 1; qq <= q; qq++)
	{
    	vector<int> tmp;
    	for (int i = 1; i <= n; i++)
    	{
    		tmp.push_back(i);
    	}
    	vector<int> hull = convexHull(tmp);
    	int a = -1, b = -1, c = -1;
    	for (int i = 1; i + 1 < (int)hull.size(); i++)
    	{
    		if (ask(hull[0], hull[i], hull[i + 1]))
    		{
    			a = hull[0], b = hull[i], c = hull[i + 1];
    			break;
    		}
    	}
			//cerr << hull.size() << endl;
			//cerr << a << " " << b << " " << c << endl;
    	while (1)
    	{
    		vector<int> cur;
    		for (int i = 1; i <= n; i++)
    		{
    			if (i != a && i != b && i != c && inside(a, b, c, i))
    			{
    				cur.push_back(i);
    			}
    		}
    		if (cur.empty())
    		{
    			cout << "! 3 " << a << " " << b << " " << c << endl;
                break;
    		}
    		int x = cur[0];
				//cerr << x << endl;
    		if (ask(a, b, x))
    		{
    			c = x;
    			continue;
    		}
    		if (ask(b, c, x))
    		{
    			a = x;
    			continue;
    		}
    		if (ask(a, c, x))
    		{
    			b = x;
    			continue;
    		}
    		assert(0);
    	}
	}
}
