#include <cstdio>
#include <vector>
#include <cassert>
#include <algorithm>
#include <set>
#include <functional>

using namespace std;

const int INF = 0x3fffffff;
typedef long long ll;

//#define TESTL

struct Linear {
  vector<ll> a;
  vector<ll> b;
  vector<ll> t;

#ifdef TESTL
  vector<ll> aa, ab;
#endif

  ll get(ll x) {
    int id = upper_bound(t.begin(), t.end(), x, greater<int>()) - t.begin();
    assert(id != 0);
    id--;
    ll res = a[id] * x + b[id];
#ifdef TESTL
    ll res2 = 0;
    int rid = -1;
    for (int i = 0; i < (int) aa.size(); i++) {
      if (res2 < aa[i] * x + ab[i]) {
        rid = i;
        res2 = aa[i] * x + ab[i];
      }
    }
    fprintf(stderr, "%p %lld => %lld == %lld?\n", this, x, res, res2);
    if (res != res2) {
      fprintf(stderr, "%lld,%lld %lld,%lld\n", a[id], b[id], aa[rid], ab[rid]);
      for (int i = 0; i < (int)a.size(); i++) {
        printf("%lld %lld %lld\n", a[i], b[i], t[i]);
      }
    }
    assert(res == res2);
#endif
    return res;
  }

  void add(ll na, ll nb) {
#ifdef TESTL
    fprintf(stderr, "%p %lld %lld\n", this, na, nb);
    aa.push_back(na);
    ab.push_back(nb);
#endif
    while (!t.empty() && na * t.back() + nb > a.back() * t.back() + b.back()) {
      a.pop_back();
      b.pop_back();
      t.pop_back();
    }

    if (t.empty()) {
      a.push_back(na);
      b.push_back(nb);
      t.push_back(INF);
      return;
    }
    assert(a.back() - na > 0);
    if (nb - b.back() < 0) {
      return;
    }

    // a1 * x + b1 > a2 * x + b2
    // (a1 - a2) * x > b2 - b1
    
    ll nt = (nb - b.back()) / (a.back() - na);
    //printf("%lld,%lld ; %lld,%lld -> %lld\n", na, nb, a.back(), b.back(), nt);
    assert(na * nt + nb >= a.back() * nt + b.back());
    assert(na * (nt + 1) + nb < a.back() * (nt + 1) + b.back());
    a.push_back(na);
    b.push_back(nb);
    t.push_back(nt);
  }

};


struct edge {
  int from, to, w, ok;
};

vector<edge> es;
vector<vector<int>> g;
vector<vector<int>> path;
vector<vector<pair<int, int>>> rpath;
vector<ll> sum;
vector<Linear> cur;

vector<int> dijkstra() {
  vector<int> d(g.size(), INF);
  d[0] = 0;
  set<pair<int, int>> s;
  s.insert({0, 0});
  while (!s.empty()) {
    int from = s.begin()->second;
    s.erase(s.begin());
    for (int e: g[from]) {
      if (!es[e].ok) continue;
      int to = es[e].to;
      int w = es[e].w;
      if (d[to] > d[from] + w) {
        s.erase({d[to], to});
        d[to] = d[from] + w;
        s.insert({d[to], to});
      }
    }
  }
  return d;
}

int main() {
  freopen("trains.in", "r", stdin);
  freopen("trains.out", "w", stdout);
  int n, p;
  scanf("%d%d", &n, &p);
  g.resize(n);
  path.resize(p);
  for (int i = 0; i < p; i++) {
    int k;
    scanf("%d", &k);
    path[i] = vector<int>(k);
    int last;
    scanf("%d", &last);
    --last;
    for (int j = 0; j < k; j++) {
      path[i][j] = es.size();
      int nextv, len;
      scanf("%d%d", &len, &nextv);
      --nextv;
      g[last].push_back(es.size());
      es.push_back({last, nextv, len, 1});
      last = nextv;
    }
  }

  vector<int> d = dijkstra();

  if (d[n - 1] == INF) {
    printf("-1\n");
    return 0;
  }


  rpath.resize(n);
  int pcnt = 0;
  for (int i = 0; i < p; i++) {
    bool in = false;
    for (int x : path[i]) {
      if (d[es[x].from] != INF && d[es[x].from] + es[x].w == d[es[x].to]) {
        if (!in) {
          in = true;
          pcnt++;
          rpath[es[x].from].push_back({pcnt - 1, -1});
        }
        rpath[es[x].to].push_back({pcnt - 1, x});
      } else {
        in = false;
      }
    }
  }

  sum.resize(pcnt);
  cur.resize(pcnt);

  vector<int> order(n);
  for (int i = 0; i < n; i++) {
    order[i] = i;
  }
  sort(order.begin(), order.end(), [&d](int a, int b) { return d[a] < d[b];});


  for (int i: order) {
    assert(d[i] != INF);

    ll bestd = 0;
    for (auto x: rpath[i]) {
      if (x.second != -1) {
        sum[x.first] += es[x.second].w;
        bestd = max(bestd, cur[x.first].get(sum[x.first]) + sum[x.first] * sum[x.first]);
      }
    }
    for (auto x: rpath[i]) {
      cur[x.first].add(-2 * sum[x.first], bestd + sum[x.first] * sum[x.first]);
    }
    if (i == n - 1) {
      printf("%d %lld\n", d[i], bestd);
      return 0;
    }
  }
  assert(0);
  return 0;
}
