/*
File name: segments_ma_nlog2n.cpp
Tag: ACCEPTED
Author: makhmedov
Change time: Thu Mar 30 01:33:17 MSK 2017

*/
#include <cstdio>
#include <memory.h>
#include <cassert>
#include <tuple>
#include <algorithm>
#include <vector>
using namespace std;

typedef long long llong;

#define eprintf(...) fprintf(stderr, __VA_ARGS__)

#ifndef LOCAL42
    #define TASK "internship"
#endif

vector<pair<int, int>> pts;

const int N = 1000500;
const llong inf = 1e16;

vector<int> A, WA;
vector<llong> SA;
vector<int> B, WB;
vector<llong> SB;

void read(int n, vector<int>& X, vector<int>& WX, vector<llong>& SX) {
    X.resize(n);
    WX.resize(n);
    SX.resize(n + 1);
    for (int i = 0; i < n; i++) {
        scanf("%d", &X[i]);
        --X[i];
    }
    for (int i = 0; i < n; i++) {
        scanf("%d", &WX[i]);
        SX[i + 1] = SX[i] + WX[i];
    }
}

struct cake {
    vector<llong> T, D;
    int TN;
    cake() {
        TN = 1;
        while (TN < (int)B.size() + 1)
            TN <<= 1;
        T.resize(2 * TN, -inf);
        D.resize(2 * TN, 0);
        for (int i = 0; i <= (int)B.size(); i++) {
            T[TN + i] += -SB[i];
        }
        for (int i = TN - 1; i > 0; i--) {
            T[i] = max(T[2 * i], T[2 * i + 1]);
        }
    }
    pair<llong, int> getMax() {
        llong mx = T[1] + D[1];
        int v = 1;
        while (v < TN) {
            if (T[2 * v] + D[2 * v] > T[2 * v + 1] + D[2 * v + 1]) {
                v <<= 1;
            } else {
                v <<= 1;
                ++v;
            }
        }
        return make_pair(mx, v - TN);
    }

    void add(int l, int r, int L, int R, int v, llong x) {
        if (r < L || l > R) {
            return;
        } else if (l <= L && R <= r) {
            D[v] += x;
        } else {
            add(l, r, L, (L + R) / 2, 2 * v, x);
            add(l, r, (L + R) / 2 + 1, R, 2 * v + 1, x);
            T[v] = max(T[2 * v] + D[2 * v], T[2 * v + 1] + D[2 * v + 1]);
        }
    }
    void add(int l, int r, llong x) {
        add(l, r, 0, TN - 1, 1, x);
    }
};

struct conqueror {
    vector<pair<int, llong>> SL = {{-1, 0ll}};
    vector<pair<int, llong>> SR = {{-1, 0ll}};
    int mid;
    int l, r;

    int last = -1;
    tuple<llong, int, int> ans;

    cake C;

    conqueror(int _mid, int _l, int _r) : mid(_mid), l(_l), r(_r)
    { }

    void feed(int x, int y, bool relaxOnly = false) {
        C.add(last + 1, y, inf + SA[r] - SA[l + 1]);
        auto cur = C.getMax();
        cur.first += SB[y];
        if (get<0>(ans) < cur.first) {
            ans = make_tuple(cur.first, cur.second, y - 1);
        }

        if (relaxOnly) {
            return;
        }

        auto& S = (x < mid) ? SL : SR;
        S.emplace_back(y, (x < mid) ? SA[mid] - SA[l + 1] : SA[r] - SA[mid]);
        llong w = (x < mid) ? (SA[mid] - SA[x + 1]) : (SA[x] - SA[mid]);
        while (S.back().second > w) {
            assert(S.size() >= 2);
            int from = S[(int)S.size() - 2].first + 1;
            int to = S[(int)S.size() - 1].first;
            C.add(from, to, -S.back().second + w);
            S.pop_back();
        }
        S.emplace_back(y, w);
        last = y;
    }
};

pair<int, int> tmp[N];

bool cmpByY(pair<int, int> a, pair<int, int> b) {
    return a.second < b.second;
}

tuple<llong, int, int> divideAndConquer(int l, int r) {
    if (l >= r) {
        return make_tuple(-inf, -1, -1);
    }
    int e = (l + r + 2) / 2 - 1;
    int midx = pts[e + 1].first;
    int lx = pts[l].first;
    int rx = pts[r].first;
    tuple<llong, int, int> res(-inf, -1, -1);
    if (l != r) {
        auto resl = divideAndConquer(l, e);
        auto resr = divideAndConquer(e + 1, r);
        res = max(resl, resr);
    }
    merge(pts.begin() + l, pts.begin() + e + 1, pts.begin() + e + 1, pts.begin() + r + 1, tmp, cmpByY);
    memcpy(&pts[l], tmp, sizeof(tmp[0]) * (r - l + 1));

    conqueror C(midx, lx, rx);
    for (int i = l; i <= r; i++) {
        if (pts[i].first >= 0 && pts[i].first < (int)A.size()) {
            const auto& pt = pts[i];
            C.feed(pt.first, pt.second);
        }
    }
    C.feed(-42, B.size(), true /* relaxOnly */);
    auto resHere = C.ans;
    return max(res, resHere);
}

tuple<llong, int, int, int, int> restoreByB(int l, int r) {
    vector<int> usedB(B.begin() + l, B.begin() + r + 1);
    sort(usedB.begin(), usedB.end());
    tuple<int, int, int> best(0, 0, -1);
    int last = -1;
    for (int i = 0; i < (int)A.size(); i++) {
        if (binary_search(usedB.begin(), usedB.end(), A[i])) {
            last = -1;
            continue;
        }
        if (last == -1) {
            last = i;
        }
        if (get<0>(best) < i - last + 1) {
            best = make_tuple(i - last + 1, last, i);
        }
    }
    int al, ar;
    tie(ignore, al, ar) = best;
    return make_tuple(SA[ar + 1] - SA[al] + SB[r + 1] - SB[l], al, ar, l, r);
}

int main() {
#ifdef TASK
    freopen(TASK ".in", "r", stdin);
    freopen(TASK ".out", "w", stdout);
#endif
    int n, m;
    scanf("%d %d", &n, &m);
    pts.resize(n + m, {-1, -1});
    read(n, A, WA, SA);
    read(m, B, WB, SB);
    for (int i = 0; i < (int)A.size(); i++) {
        pts[A[i]].first = i;
    }
    for (int i = 0; i < (int)B.size(); i++) {
        pts[B[i]].second = i;
    }
    pts.erase(remove_if(pts.begin(), pts.end(), [] (pair<int, int> pt) { return pt.first == -1 || pt.second == -1; }), pts.end());
    sort(pts.begin(), pts.end());
    pts.insert(pts.begin(), make_pair(-1, -42));
    pts.insert(pts.end(), make_pair(A.size(), -42));

    auto ans = divideAndConquer(0, (int)pts.size() - 1);
    auto res = restoreByB(get<1>(ans), get<2>(ans));
    llong totw;
    int l0, r0, l1, r1;
    tie(totw, l0, r0, l1, r1) = res;
    l0++, r0++, l1++, r1++;
    if (r0 == 0)
        l0 = 0;
    if (r1 == 0)
        l1 = 0;
    printf("%lld\n%d %d\n%d %d\n", totw, l0, r0, l1, r1);
}

