#include <bits/stdc++.h>

#define endl "\n"

using namespace std;

const int INF = (int) 1e9 + 3;

struct Treap {
    struct Node {
        Node(int l, int r)
                : size(1), priority(rand()), l(l), r(r), maxl(l), minr(r) {}

        Node *left = nullptr, *right = nullptr;
        int size;
        int priority, l, r, maxl, minr;


        void update() {
            int cmaxl, cminr;
            tie(cmaxl, cminr) = get_bounds(this->left);
            maxl = max(l, cmaxl);
            minr = min(r, cminr);

            tie(cmaxl, cminr) = get_bounds(this->right);
            maxl = max(maxl, cmaxl);
            minr = min(minr, cminr);

            size = get_size(this->left) + get_size(this->right) + 1;
        }
    };

    static pair<int, int> get_bounds(Node *node) {
        if (node == nullptr) {
            return {-INF, INF};
        } else {
            return {node->maxl, node->minr};
        }
    }

    static int get_size(Node *node) {
        return node == nullptr ? 0 : node->size;
    }

    Node *root = nullptr;

    void add(int l, int r) {
        root = merge(root, new Node(l, r));
    }

    pair<int, int> get(int l, int r) {
        Node *left, *middle, *right;
        tie(left, right) = split(root, r);
        tie(left, middle) = split(left, l);
        auto res = get_bounds(middle);
        root = merge(merge(left, middle), right);
        return res;
    };

    pair<Node *, Node *> split(Node *root, int bound) {
        if (root == nullptr) {
            return {nullptr, nullptr};
        }

        if (get_size(root->left) >= bound) {
            Node *left;
            tie(left, root->left) = split(root->left, bound);
            root->update();
            return {left, root};
        } else {
            Node *right;
            tie(root->right, right) = split(root->right, bound - get_size(root->left) - 1);
            root->update();
            return {root, right};
        }
    }

    Node *merge(Node *left, Node *right) {
        if (left == nullptr) {
            return right;
        }
        if (right == nullptr) {
            return left;
        }

        if (left->priority > right->priority) {
            left->right = merge(left->right, right);
            left->update();
            return left;
        } else {
            right->left = merge(left, right->left);
            right->update();
            return right;
        }
    }
};

void solve() {
    int n;
    cin >> n;
    vector<int> times(n);
    for (int i = 0; i < n; ++i) {
        cin >> times[i];
    }
    vector<int> ls, rs;
    Treap lefts, rights;
    for (int i = 0; i < n - 1; ++i) {
        int l, r;
        cin >> l >> r;
        ++r;
        lefts.add(max(l - times[i + 1], 0), r);
        rights.add(max(l - times[i], 0), r);
        ls.emplace_back(l);
        rs.emplace_back(r);
    }

    for (int i = 0; i < n; ++i) {
        int l1, r1, l2, r2;
        tie(l1, r1) = lefts.get(0, i);
        tie(l2, r2) = rights.get(i, n - 1);

        bool is_bad = false;
        int maxl = -INF;
        for (int j = i - 1; j >= 0; --j) {
            if (maxl >= rs[j]) {
                is_bad = true;
                break;
            }
            maxl = max(maxl, ls[j]);
        }
        maxl = -INF;
        for (int j = i + 1; j < n - 1; ++j) {
            if (maxl >= rs[j]) {
                is_bad = true;
                break;
            }
            maxl = max(maxl, ls[j]);
        }

        if (max(l1, l2) < min(r1, r2) and not is_bad) {
            cout << max(max(l1, l2), 0) << endl;
        } else {
            cout << "-1" << endl;
        }
    }
}

int main() {
#ifdef MINE
    freopen("../input.txt", "r", stdin);
#endif
    ios_base::sync_with_stdio(false);
    cin.tie(nullptr);

    solve();

    return 0;
}