#include <bits/stdc++.h>

using namespace std;

struct Point {
    long long x, y;
    int id;

    Point(long long x, long long y, int id) : x(x), y(y), id(id) {}

    Point() {}

    Point(Point &a, Point &b) : x(b.x - a.x), y(b.y - a.y), id(-1) {}
};

long long cross(Point a, Point b) {
    return a.x * b.y - a.y * b.x;
}

long long dist(Point a, Point b) {
    return (a.x - b.x) * (a.x - b.x) + (a.y - b.y) * (a.y - b.y);
}

long long dot(Point a, Point b) {
    return a.x * b.x + a.y * b.y;
}

int n;
vector<Point> coords;
vector<Point> chull;
Point st;

bool cmp(Point &a, Point &b) {
    long long c = cross(Point(st, a), Point(st, b));
    if (c != 0) return c > 0;
    return dist(st, a) < dist(st, b);
}

vector<Point> convex_hull() {
    st = coords[0];
    for (int i = 0; i < coords.size(); i++) {
        if (coords[i].y < st.y || (coords[i].y == st.y && coords[i].x < st.x)) {
            st = coords[i];
        }
    }
    sort(coords.begin(), coords.end(), cmp);

    vector<Point> ans;
    for (int i = 0; i < coords.size(); i++) {
        while (ans.size() >= 2 && cross(Point(ans[ans.size() - 2], ans[ans.size() - 1]),
                                        Point(ans[ans.size() - 1], coords[i])) <= 0) {
            ans.pop_back();
        }
        ans.push_back(coords[i]);
    }

    return ans;
}

long double angle(Point a, Point b) {
    return atan2(cross(a, b), dot(a, b));
}

bool is_in(Point &p, Point &a, Point &b, Point &c) {
    // Point p is in the triangle {a, b, c}
    long double sm = 0;
    sm += angle(Point(p, a), Point(p, b));
    sm += angle(Point(p, b), Point(p, c));
    sm += angle(Point(p, c), Point(p, a));

    return abs(sm) > 0.1;
}

void find() {
    // In polygon {0, L, L + 1, L + 2, ..., R - 1, R} there is a tiger
    int L = 1;
    int R = chull.size() - 1;
    while (R - L > 1) {
        int M = (L + R) / 2;

        cout << "? " << M - L + 2 << " " << chull[0].id;
        for (int i = L; i <= M; i++) {
            cout << " " << chull[i].id;
        }
        cout << endl;

        string ans;
        cin >> ans;

        if (ans == "Yes") {
            R = M;
        } else {
            L = M;
        }
    }

    // Tiger is in triangle {0, L, L + 1}
    Point a = chull[0];
    Point b = chull[L];
    Point c = chull[L + 1];
//    cout << "OK " << a.id << " " << b.id << " " << c.id << endl;

    vector<Point> pts;
    for (Point p : coords) {
        if (p.id != a.id && p.id != b.id && p.id != c.id && is_in(p, a, b, c)) {
            pts.push_back(p);
        }
    }

    while (pts.size() > 0) {

        int best = 1000000000;
        Point bp = pts[0];

        for (auto pp : pts) {
            int cntab = 0, cntac = 0, cntbc = 0;
            for (auto p : pts) {
                if (p.id == pp.id) continue;
                if (is_in(a, b, pp, p)) {
                    cntab++;
                } else if (is_in(b, c, pp, p)) {
                    cntbc++;
                } else if (is_in(c, a, pp, p)) {
                    cntac++;
                } else {
                    exit(239566);
                }
            }

            int mx = max(max(cntab, cntbc), cntac);

            if (mx < best) {
                best = mx;
                bp = pp;
            }
        }

        string ans;

        cout << "? 3 " << a.id << " " << b.id << " " << bp.id << endl;
        cin >> ans;
        if (ans == "Yes") {
            c = bp;
        } else {
            cout << "? 3 " << b.id << " " << c.id << " " << bp.id << endl;
            cin >> ans;
            if (ans == "Yes") {
                a = bp;
            } else {
                b = bp;
            }
        }

        vector<Point> newpts;
        for (auto p : pts) {
            if (p.id != a.id && p.id != b.id && p.id != c.id && is_in(a, b, c, p)) {
                newpts.push_back(p);
            }
        }

        pts.swap(newpts);
    }

    cout << "! 3 " << a.id << " " << b.id << " " << c.id << endl;
}

int main() {
    ios::sync_with_stdio(0);
    cin.tie(0);

    cin >> n;
    coords.resize(n);
    for (int i = 0; i < n; i++) {
        cin >> coords[i].x >> coords[i].y;
        coords[i].id = i + 1;
    }
    chull = convex_hull();

//    cout << "CONVEX HULL : ";
    for (auto p : chull) {
        cout << p.id << " ";
    }
    cout << endl;

    int q;
    cin >> q;
    for (int i = 0; i < q; i++) {
        find();
    }

    return 0;
}

/*

6
3 0
0 2
2 4
4 2
5 5
6 1
4

 */