#include <iostream>
#include <algorithm>
#include <cstdio>
#include <set>
#include <cassert>
#include <vector>
#include <cmath>

using namespace std;

const double eps = 1e-8;

bool EQ(double a, double b) { return abs(a - b) <= eps; }

#define int int64_t

struct Point {
    int x, y;
    int num;
    Point(int x = 0, int y = 0, int num = -1):
        x(x), y(y), num(num) {}
};

int sqr(int a) { return a * a; }

Point o;
bool compare_Points(Point p1, Point p2) {
    if (p1.x == o.x && p1.y == o.y)
        return true;
    if (p2.x == o.x && p2.y == o.y)
        return false;
    p1.x -= o.x, p2.x -= o.x;
    p1.y -= o.y, p2.y -= o.y;
    if (p1.y * p2.x == p2.y * p1.x)
        return sqr(p1.x) + sqr(p1.y) < sqr(p2.x) + sqr(p2.y);
    return p1.y * p2.x < p2.y * p1.x;
}

struct Vector {
    int x, y;
    Vector(int x = 0, int y = 0):
        x(x), y(y) {}
    Vector(Point p1, Point p2):
        x(p2.x - p1.x), y(p2.y - p1.y) {}
};

int operator ^(Vector a, Vector b) { return a.x * b.y - a.y * b.x; }
int operator *(Vector a, Vector b) { return a.x * b.x + a.y * b.y; }

vector <Point> get_hull(vector <Point> p) {
    if ((int)p.size() <= 3)
        return p;
    o = p[0];
    for (int i = 1; i < (int)p.size(); i++)
        if (make_pair(o.x, o.y) > make_pair(p[i].x, p[i].y))
            o = p[i];
    sort(p.begin(), p.end(), compare_Points);

    vector <Point> ans;
    ans.push_back(p[0]);
    ans.push_back(p[1]);
    for (int i = 2; i < (int)p.size(); i++) {
        while ((int)ans.size() >= 2 && (Vector(ans.back(), ans[(int)ans.size() - 2]) ^ Vector(ans.back(), p[i])) > 0)
            ans.pop_back();
        ans.push_back(p[i]);
    }

    return ans;
}

bool get(vector <Point> p) {
    if ((int)p.size() < 3)
        return false;
    cout << "? ";
    cout << (int)p.size() << " ";
    for (int i = 0; i < (int)p.size(); i++)
        cout << p[i].num + 1 << " ";
    cout << endl;
    string s; cin >> s;
    return s == "Yes";
}

vector <Point> submas(vector <Point> &p, int l, int r) {
    vector <Point> ans;
    for (int i = l; i <= r; i++)
        ans.push_back(p[i]);
    return ans;
}

vector <Point> p;

double angle(Vector v1, Vector v2) { return atan2(v1 ^ v2, v1 * v2); }

bool in(Point a, Point b, Point c, Point p) {
    return !EQ(angle(Vector(p, a), Vector(p, b)) + angle(Vector(p, b), Vector(p, c)) + angle(Vector(p, c), Vector(p, a)), 0);
}

int s(Point a, Point b, Point c) { return abs(Vector(a, b) ^ Vector(a, c)); }

bool solve(vector <Point> p) {
    vector <Point> h = get_hull(p);
    if (!get(h))
        return false;

    int left = 1, right = (int)h.size();
    while (right - left > 2) {
        int mid = (left + right) / 2;
        vector <Point> g = submas(h, mid, right - 1);
        g.push_back(h[0]);
        if (get(g))
            left = mid;
        else
            right = mid + 1;
    }

    vector <Point> nxt;
    Point c = h[0], b = h[left], a = h[left + 1];
    for (int i = 0; i < (int)p.size(); i++)
        if ((p[i].x != c.x || p[i].y != c.y) && in(a, b, c, p[i]))
            nxt.push_back(p[i]);

    if ((int)nxt.size() == 2) {
        cout << "! 3 " << a.num + 1 << " " << b.num + 1 << " " << c.num + 1 << endl;
        return true;
    }
    if (solve(nxt))
        return true;

    h = get_hull(nxt);

    int mins = 1e18;
    for (int i = 0; i < (int)h.size(); i++) {
        int j = (i + 1) % (int)h.size();
        vector <Point> w = {h[i], h[j], c};
        if (mins > s(h[i], h[j], c) && get(w))
            mins = s(h[i], h[j], c), a = h[i], b = h[j];
    }

    cout << "! 3 " << a.num + 1 << " " << b.num + 1 << " " << c.num + 1 << endl;
    return true;
}

int32_t main() {
    //assert(freopen("input.txt", "r", stdin));
    //assert(freopen("output.txt", "w", stdout));

    ios_base::sync_with_stdio(false);

    int n; cin >> n;
    p.resize(n);
    for (int i = 0; i < n; i++) {
        cin >> p[i].x >> p[i].y;
        p[i].num = i;
    }

    int q; cin >> q;

    for (int i = 0; i < q; i++)
        solve(p);

    return 0;
}
