#include <iostream>
#include <vector>
#include <algorithm>
#include <map>
#include <cmath>
#include <set>

using namespace std;

constexpr int maxn = 2e4;

using ll = long long;
using pii = pair<ll, ll>;
pii v[maxn];
constexpr int inf = 1e9+20;
using ld = long double;

#define mp make_pair
#define fst first
#define snd second
#define sz(a_) int((a_).size())

vector<pii> convex_hull(vector<pii> v)
{
    int mie = min_element(v.begin(), v.end())-v.begin();
    int mae = max_element(v.begin(), v.end())-v.begin();
    vector<pii> res;
    auto cur = v[mie];
    for (;;) {
        res.push_back(cur);
        pii best = mp(inf, inf);
        for (auto p: v) {
            if (p.fst > cur.fst && (best.fst == inf ||
                                    1.*(p.snd-cur.snd)/(p.fst-cur.fst) <
                                    1.*(best.snd-cur.snd)/(best.fst-cur.fst))) {
                best = p;
            }
        }
        if (best.fst == inf || best == v[mae]) {
            break;
        }
        cur = best;
    }
    cur = v[mae];
    for (;;) {
        res.push_back(cur);
        pii best = mp(inf, inf);
        for (auto p: v) {
            if (p.fst < cur.fst && (best.fst == inf ||
                                    1.*(p.snd-cur.snd)/(p.fst-cur.fst) <
                                    1.*(best.snd-cur.snd)/(best.fst-cur.fst))) {
                best = p;
            }
        }
        if (best.fst == inf || best == v[mie]) {
            break;
        }
        cur = best;
    }
    return res;
}

vector<vector<pii>> trs;

map<pii, int> ma;

ld dist(pii a, pii b)
{
    return hypot(a.fst-b.fst, a.snd-b.snd);
}

ld area(pii a, pii b, pii c)
{
    ld la = dist(a, b);
    ld lb = dist(b, c);
    ld lc = dist(a, c);
    ld hp = (la+lb+lc)/2;
    return sqrt(hp*(hp-la)*(hp-lb)*(hp-lc));
}

bool inside(pii a, pii b, pii c, pii q)
{
    return abs(area(a, b, q)+area(b, c, q)+area(a, c, q)-area(a, b, c)) < 1e-7;
}

void br(vector<pii> ps)
{
    if (ps.size() == 3) {
        trs.push_back(ps);
        return;
    }
    auto u = convex_hull(ps);
    set<pii> e(u.begin(), u.end());
    vector<vector<pii>> g(sz(u)-2);
    for (int i = 1; i+1 < sz(u); ++i) {
        for (auto p: ps) {
            if (inside(u[0], u[i], u[i+1], p)) {
                g[i-1].push_back(p);
            }
        }
        br(g[i-1]);
    }
}

int main()
{
    int n;
    cin >> n;
    map<pii, int> ma;
    for (int i = 0; i < n; ++i) {
        cin >> v[i].fst >> v[i].snd;
        ma[v[i]] = i+1;
    }


    int q;
    cin >> q;
    string ans;
    for (int w = 0; w < q; ++w) {
        for (auto &tr: trs) {
            cout << "? " << tr.size() << ' ';
            for (auto e: tr) {
                cout << ma[e] << ' ';
            }
            cout << endl;
            cin >> ans;
            if (ans[0] == 'Y') {
                cout << "! " << tr.size() << ' ';
                for (auto e: tr) {
                    cout << ma[e] << ' ';
                }
                cout << endl;
                break;
            }
        }
    }
    return 0;
}