#include <iostream>
#include <vector>
#include <algorithm>
#include <map>
#include <cmath>
#include <set>

using namespace std;

constexpr int maxn = 2e4;

int x[maxn];
int y[maxn];
using pii = pair<int, int>;
constexpr int inf = 1e9+20;
using ld = long double;

#define mp make_pair
#define fst first
#define snd second
#define sz(a_) int((a_).size())

vector<pii> convex_hull(vector<pii> v)
{
    int mie = min_element(v.begin(), v.end())-v.begin();
    int mae = max_element(v.begin(), v.end())-v.begin();
    vector<pii> res;
    auto cur = v[mie];
    for (;;) {
        res.push_back(cur);
        pii best = mp(inf, inf);
        for (auto p: v) {
            if (p.fst > cur.fst && (best.fst == inf ||
                    1.*(p.snd-cur.snd)/(p.fst-cur.fst) <
                                   1.*(best.snd-cur.snd)/(best.fst-cur.fst))) {
                best = p;
            }
        }
        if (best.fst == inf || best == v[mae]) {
            break;
        }
        cur = best;
    }
    cur = v[mae];
    for (;;) {
        res.push_back(cur);
        pii best = mp(inf, inf);
        for (auto p: v) {
            if (p.fst < cur.fst && (best.fst == inf ||
                                    1.*(p.snd-cur.snd)/(p.fst-cur.fst) <
                                   1.*(best.snd-cur.snd)/(best.fst-cur.fst))) {
                best = p;
            }
        }
        if (best.fst == inf || best == v[mie]) {
            break;
        }
        cur = best;
    }
    return res;
}

map<pii, int> ma;

ld dist(pii a, pii b)
{
    return hypot(a.fst-b.fst, a.snd-b.snd);
}

ld area(pii a, pii b, pii c)
{
    ld la = dist(a, b);
    ld lb = dist(b, c);
    ld lc = dist(a, c);
    ld hp = (la+lb+lc)/2;
    return sqrt(hp*(hp-la)*(hp-lb)*(hp-lc));
}

bool inside(pii a, pii b, pii c, pii q)
{
    return abs(area(a, b, q)+area(b, c, q)+area(a, c, q)-area(a, b, c)) < 1e-7;
}

void solve2(vector<pii> le, vector<pii> ri)
{
    int lp = sz(le)-1, rp = sz(ri)-1;
    vector<pair<pii, pii>> q{mp(le.back(), ri.back())};
    while (lp > 1 || rp > 1) {
        if (inside(ri[rp-1], le[lp], ri[rp], le[lp-1])) {
            q.push_back(mp(le[lp-1], ri[rp]));
            --lp;
        } else {
            q.push_back(mp(le[lp], ri[rp-1]));
            --rp;
        }
    }
    if (lp == 1) {
        while (rp > 0) {
            q.push_back(mp(le[lp], ri[rp-1]));
            --rp;
        }
    } else if (rp == 1) {
        while (lp > 0) {
            q.push_back(mp(le[lp-1], ri[rp]));
            --lp;
        }
    }
    int l = 0, r = sz(q)-1;
    string ans;
    while (r-l > 1) {
        int m = (l+r)/2;
        cout << "? " << ma[le[0]]+1 << ' ' << ma[q[m].fst]+1 << ' ' << ma[q[m].snd]+1 << endl;
        cin >> ans;
        if (ans[0] == 'Y') {
            l = m;
        } else {
            r = m;
        };
    }
    set<pii> o{q[l].fst, q[l].snd, q[l+1].fst, q[l+1].snd};
    vector<pii> pe(o.begin(), o.end());
    cout << "! 3 " << ma[pe[0]]+1 << ' ' << ma[pe[1]]+1 << ' ' << ma[pe[2]]+1 << endl;
}

void solve(vector<pii> cur)
{
    if (cur.size() == 3) {
        cout << "! 3 " <<
             ma[cur[0]]+1 << ' ' <<
             ma[cur[1]]+1 << ' ' <<
             ma[cur[2]]+1 << endl;
        return;
    }
    vector<pii> tmp(cur.begin()+1, cur.end());
    sort(tmp.begin(), tmp.end(), [&cur](const pii& a, const pii& b){
        return atan2(a.snd-cur[0].snd, a.fst-cur[0].fst) < atan2(b.snd-cur[0].snd, b.fst-cur[0].fst);
    });
    vector<pii> up{cur[0]}, down{cur[0]};
    for (int i = 0; i < sz(tmp); ++i) {
        if (i <= (sz(tmp)-1)/2) {
            down.push_back(tmp[i]);
        }
        if (i >= (sz(tmp)-1)/2) {
            up.push_back(tmp[i]);
        }
    }
    auto cd = convex_hull(down), cu = convex_hull(up);
    cout << "? " << sz(cd) << ' ';
    for (auto p: cd) {
        cout << ma[p]+1 << ' ';
    }
    cout << endl;
    string res;
    cin >> res;
    if (res[0] == 'Y') {
        solve(down);
        return;
    }
    cout << "? " << sz(cu) << ' ';
    for (auto p: cu) {
        cout << ma[p]+1 << ' ';
    }
    cout << endl;
    cin >> res;
    if (res[0] == 'Y') {
        solve(up);
        return;
    }
    auto th = convex_hull(cur);
    set<pii> sh(th.begin(), th.end());
    int sup = -1;
    for (int i = 0; i < sz(cu); ++i) {
        if (cu[i] == cur[0]) {
            sup = (i+1)%sz(cu);
        }
    }
    vector<pii> ru;
    for (int i = sup; ; i = (i+1)%sz(cu)) {
        ru.push_back(cu[i]);
        if (sh.count(cu[i]) > 0) {
            break;
        }
    }
    vector<pii> lu;
    for (int i = (sup-1+sz(cu))%sz(cu); ; i = (i-1+sz(cu))%sz(cu)) {
        lu.push_back(cu[i]);
        if (sh.count(cu[i]) > 0) {
            break;
        }
    }
    int dup = -1;
    for (int i = 0; i < sz(cd); ++i) {
        if (cd[i] == cur[0]) {
            dup = i%sz(cd);
        }
    }
    vector<pii> ld;
    for (int i = dup; ; i = (i+1)%sz(cd)) {
        ld.push_back(cd[i]);
        if (sh.count(cd[i]) > 0) {
            break;
        }
    }
    vector<pii> rd;
    for (int i = (dup-1+sz(cd))%sz(cd); ; i = (i-1+sz(cd))%sz(cd)) {
        rd.push_back(cd[i]);
        if (sh.count(cd[i]) > 0) {
            break;
        }
    }
    vector<pii> le, ri;
    if (sz(ru) > 1 && sz(lu) > 1) {
        cout << "? " << ma[ru[0]]+1 << ' ' << ma[ru.back()]+1 << ' ' << ma[rd.back()]+1 << endl;
        string ans;
        cin >> ans;
        if (ans[0] == 'Y') {
            le = ru;
            ri = rd;
        } else {
            le = ld;
            ri = lu;
        }
    } else if (sz(ru) > 1) {
        le = ru;
        ri = rd;
    } else {
        le = ld;
        ri = lu;
    }
    solve2(le, ri);
}

int main()
{
    ios_base::sync_with_stdio(false);
    cin.tie(nullptr);
    cout.tie(nullptr);
    int n;
    cin >> n;
    for (int i = 0; i < n; ++i) {
        cin >> x[i] >> y[i];
    }
    int q;
    cin >> q;
    for (int w = 0; w < q; ++w) {
        vector<pii> start;
        ma.clear();
        for (int i = 0; i < n; ++i) {
            start.push_back(mp(x[i], y[i]));
            ma[mp(x[i], y[i])] = i;
        }
        solve(start);
    }
    return 0;
}