#include <iostream>
#include <vector>
#include <algorithm>
#include <string>
#include <cmath>
#include <set>

using namespace std;

struct point {
	long long x = 0;
	long long y = 0;
	point(long long xx = 0, long long yy = 0) {
		x = xx;
		y = yy;
	}
};

point make_vect(point a, point b) {
	return point(b.x - a.x, b.y - a.y);
}

bool operator ==(point a, point b) {
	return make_pair(a.x, a.y) == make_pair(b.x, b.y);
}

long long vect(point a, point b) {
	return a.x * b.y - a.y * b.x;
}

long long scal(point a, point b) {
	return a.x * b.x + a.y * b.y;
}

point p0;
vector<point> a;

bool cmp1(point a1, point b1) {
	return make_pair(a1.x, a1.y) < make_pair(b1.x, b1.y);
}

bool cmp2(int i, int j) {
	point p1 = make_vect(p0, a[i]);
	point p2 = make_vect(p0, a[j]);
	return vect(p1, p2) > 0;
}

vector<int> hull() {
	int ind = min_element(a.begin(), a.end(), cmp1) - a.begin();
	p0 = a[ind];
	vector<int> b;
	for (int i = 0; i < (int)a.size(); i++) {
		if (i == ind) continue;
		b.push_back(i);
	}
	sort(b.begin(), b.end(), cmp2);
	vector<int> st;
	st.push_back(ind);
	for (int i = 0; i < (int)b.size(); i++) {
		while (st.size() > 1) {
			int p1 = st[st.size() - 2];
			int p2 = st.back();
			int p3 = b[i];
			point v1 = make_vect(a[p1], a[p2]);
			point v2 = make_vect(a[p2], a[p3]);
			if (vect(v1, v2) > 0) {
				break;
			}
			else {
				st.pop_back();
			}
		}
		st.push_back(b[i]);
	}
	return st;
}

bool in(vector<int>& h, point p) {
	double angle = 0;
	for (int i = 1; i < (int)h.size(); i++) {
		point v1 = make_vect(p, a[h[i - 1]]);
		point v2 = make_vect(p, a[h[i]]);
		angle += atan2(vect(v1, v2), scal(v1, v2));
	}
	angle += atan2(vect(make_vect(p, a[h.back()]), make_vect(p, a[h[0]])),
		scal(make_vect(p, a[h.back()]), make_vect(p, a[h[0]])));
	if (abs(angle) > 1) {
		return true;
	}
	return false;
}

int main()
{
	ios::sync_with_stdio(false);
	cin.tie(0);
	cout.tie(0);
	int n;
	cin >> n;
	a.resize(n);
	for (int i = 0; i < n; i++) {
		cin >> a[i].x >> a[i].y;
	}
	int q;
	cin >> q;
	for (int z = 0; z < q; z++) {
		vector<int> h = hull();
		set<int> ok;
		for (int i = 0; i < n; i++) {
			ok.insert(i);
		}
		while (true) {
			int inp = h[0];
			while (!ok.empty()) {
				int i = *(ok.begin());
				if (find(h.begin(), h.end(), i) != h.end()) {
					ok.erase(ok.begin());
					continue;
				}
				if (!in(h, a[i])) {
					ok.erase(ok.begin());
					continue;
				}
				inp = i;
				break;
			}
			if (inp == h[0]) {
				cout << "! ";
				cout << (int)h.size() << ' ';
				for (int i = 0; i < (int)h.size(); i++) {
					cout << h[i] + 1 << ' ';
				}
				cout << endl;
				break;
			}
			else {
				bool f = false;
				for (int i = 1; i < (int)h.size(); i++) {
					cout << "? 3 " << h[i - 1] + 1 << ' ' << h[i] + 1 << ' ' << inp + 1 << endl;
					string ans;
					cin >> ans;
					if (ans == "Yes") {
						h = { h[i - 1], h[i], inp };
						f = true;
						break;
					}
				}
				if (!f) {
					cout << "? 3 " << h.back() + 1 << ' ' << h[0] + 1 << ' ' << inp + 1 << endl;
					string ans;
					cin >> ans;
					if (ans == "Yes") {
						h = { h.back(), h[0], inp };
					}
				}
			}
		}
	}
	return 0;
}

