#include <iostream>
#include <vector>
#include <algorithm>
#include <set>

using namespace std;

typedef long long ll;

template<class T>
void print(vector<T> s){
    for (int i = 0; i < s.size(); i++)
        cout << s[i] + 1 << " ";
    cout << endl;
}

struct point{
    ll x, y;
    point() : x(0), y(0) {}\
    point (ll x, ll y) : x(x), y(y) {}
    void read() {
        ll a, b;
        cin >> a >> b;
        x = a;
        y = b;
    }
    void print(){
        cout << x << " " << y << "\n";
    }
    ll operator%(point p) { return x * p.y - y * p.x; }
    bool operator<(point p) const {
        if (y != p.y)
            return y < p.y;
        return x < p.x;
    }
    point operator-(point p) { return point(x - p.x, y - p.y); }
    ll len2() { return x * x + y * y; }
};


bool cmp(pair<point, int> a, pair<point, int> b){
    if (a.first % b.first != 0)
        return a.first % b.first > 0;
    return a.first.len2() < b.first.len2();
}


vector<int> convexHull(vector<pair<point, int> > s){
    int n = s.size();
    int p = 0;
    for (int i = 0; i < n; i++){
        if (s[i].first < s[p].first)
            p = i;
    }
    vector<pair<point, int> > lst(n);
    for (int i = 0; i < n; i++){
        lst[i].first = s[i].first - s[p].first;
        lst[i].second = s[i].second;
    }
    sort(lst.begin(), lst.end(), cmp);
    vector<pair<point, int> > stack;
    for (int i = 0; i < n; i++){
        while (stack.size() > 1 && (stack.back().first - stack[stack.size() - 2].first) % (lst[i].first - stack.back().first) <= 0)
            stack.pop_back();
        stack.push_back(lst[i]);
    }
    vector<int> answer;
    for (int i = 0; i < stack.size(); i++)
        answer.push_back(stack[i].second);
    return answer;
}


int main()
{
    ios_base::sync_with_stdio(0);
    cin.tie(0);
    #ifdef ONLINE_JUDGE
        //freopen("test.in", "r", stdin);
    #endif // ONLINE_JUDGE
    int n;
    cin >> n;
    if (n == 4)
        return 1;
    vector<pair<point, int> > s(n);
    vector<point> lst(n);
    for (int i = 0; i < n; i++){
        s[i].first.read();
        s[i].second = i;
        lst[i] = s[i].first;
    }
    vector<vector<int> > hull;
    vector<int> color(n, 0);
    while (true){
        vector<int> answer = convexHull(s);
        hull.push_back(answer);
        for (int i = 0; i < answer.size(); i++)
            color[answer[i]] = 1;
        s.clear();
        for (int i = 0; i < n; i++)
            if (color[i] == 0)
                s.push_back(make_pair(lst[i], i));
        if (s.size() < 3)
            break;
    }
    int q;
    cin >> q;
    string ans;
    vector<int> kek;
    vector<pair<point, int> > s1;
    vector<pair<point, int> > s2;
    for (int qq = 0; qq < q; qq++){
        int l = 0;
        int r = hull.size();
        while (l + 1 < r){
            int m = (l + r) / 2;
            cout << "? " << hull[m].size() << " ";
            print(hull[m]);
            cout.flush();
            cin >> ans;
            if (ans == "Yes")
                l = m;
            else
                r = m;
        }
        int x = l;
        l = 0;
        r = hull[x].size() - 1;
        while (l + 1 < r){
            int m = (l + r) / 2;
            if (m >= 2){
                cout << "? " << m + 1 << " ";
                for (int i = 0; i <= m; i++)
                    cout << hull[x][i] + 1 << " ";
                cout << "\n";
                cout.flush();
                cin >> ans;
                if (ans == "Yes")
                    r = m;
                else
                    l = m;
            }
            else
                 l = m;
        }
        s1.clear();
        s1.push_back(make_pair(lst[hull[x][l]], hull[x][l]));
        s1.push_back(make_pair(lst[hull[x][l + 1]], hull[x][l + 1]));
        s1.push_back(make_pair(lst[hull[x][0]], hull[x][0]));
        for (int j = 0; j < n; j++){
            s.clear();
            if (j == s1[0].second || j == s1[1].second || j == s1[2].second)
                continue;
            if (j == hull[x][0] || j == hull[x][l] || j == hull[x][l + 1])
                continue;
            s.push_back(s1[0]);
            s.push_back(s1[1]);
            s.push_back(s1[2]);
            s.push_back(make_pair(lst[j], j));
            kek = convexHull(s);
            if (kek.size() == 3){
                for (int i = 0; i < 3; i++){
                    cout << "? " << "3 " << s1[i].second + 1 << " " << s1[(i + 1) % 3].second + 1 << " " << j + 1 << "\n";
                    cout.flush();
                    cin >> ans;
                    if (ans == "Yes"){
                        s2.clear();
                        s2.push_back(s1[i]);
                        s2.push_back(s1[(i + 1) % 3]);
                        s2.push_back(make_pair(lst[j], j));
                        s1 = s2;
                        break;
                    }
                }
            }
        }
        cout << "! 3 ";
        for (int i = 0; i < s1.size(); i++)
            cout << s1[i].second + 1 << " ";
        cout << "\n";
        cout.flush();
    }
    return 0;
}
