#include <bits/stdc++.h>

using namespace std;

const int maxN = 5005;
const int INF = 1e9;

#define ll long long

struct Point
{
    ll x, y;
    int i;
    Point() {}
    Point(ll _x, ll _y, int _i) : x(_x), y(_y), i(_i) {}
};

Point operator - (Point p1, Point p2)
{
    return Point(p1.x - p2.x, p1.y - p2.y, p1.i);
}

Point operator + (Point p1, Point p2)
{
    return Point(p1.x + p2.x, p1.y + p2.y, p1.i);
}

ll ve(Point p1, Point p2)
{
    return p1.x * p2.y - p2.x * p1.y;
}

ll sk(Point p1, Point p2)
{
    return p1.x * p2.x + p1.y * p2.y;
}

ll dist2(Point p)
{
    return p.x * p.x + p.y * p.y;
}

ll sgn(ll x)
{
    return x < 0 ? -1 : (x == 0 ? 0 : 1);
}

Point arr[maxN];

int check(Point p, vector<Point>& ps)
{
    int was1 = 0, was2 = 0;

    for(int i = 0; i < ps.size(); i++)
    {
        if(ve(p - ps[i], ps[(i + 1) % ps.size()] - ps[i]) > 0) was1 = 1;
        else was2 = 1;
    }

    if(was1 && was2) return 0;
    return 1;
}

struct Node2
{
    Node2* n1, *n2, *n3;
    Point p, p1, p2, p3;
};

struct Node
{
    vector<Point> ps1;
    Node* left, *right;
    Node2* nxt;
};

vector<Point> buildHull(vector<Point> ps)
{
    Point p0 = ps[0];
    for(auto p : ps) if(p.x < p0.x || p.x == p0.x && p.y < p0.y) p0 = p;
    for(int i = 0; i < ps.size(); i++) ps[i] = ps[i] - p0;

    sort(ps.begin(), ps.end(), [](Point p1, Point p2){return ve(p1, p2) > 0;});

    vector<Point> st;
    for(int i = 0; i < ps.size(); i++)
    {
        while(st.size() > 2 && ve(st.back() - st[st.size() - 2], ps[i] - st.back()) < 0) st.pop_back();
        st.push_back(ps[i]);
    }

    for(int i = 0; i < st.size(); i++) st[i] = st[i] + p0;
    return st;
}

Node2* build2(vector<Point>& ps)
{
    auto v = buildHull(ps);

    Node2* node = new Node2;
    node->p1 = v[0];
    node->p2 = v[1];
    node->p3 = v[2];
    node->n1 = 0;
    node->n2 = 0;
    node->n3 = 0;

    vector<Point> v1(3), v2(3), v3(3);

    v1[0] = v2[0] = v3[0] = node->p;
    v1[1] = v3[1] = node->p1;
    v1[2] = v2[1] = node->p2;
    v2[2] = v3[2] = node->p3;

    if(ps.size() == 3) return node;

    int cnt = INF;
    for(int i = 0; i < ps.size(); i++)
    {
        Point p = ps[i];
        int c1 = 0, c2 = 0, c3 = 0;
        for(int j = 0; j < ps.size(); j++)
        {
            if(check(ps[j], v1)) c1++;
            if(check(ps[j], v2)) c2++;
            if(check(ps[j], v3)) c3++;
        }

        if(cnt > max(c1, max(c2, c3))) cnt = max(c1, max(c2, c3)), node->p = p;
    }

    vector<Point> ps1, ps2, ps3;

    for(int j = 0; j < ps.size(); j++)
    {
        if(check(ps[j], v1)) ps1.push_back(ps[j]);
        if(check(ps[j], v2)) ps2.push_back(ps[j]);
        if(check(ps[j], v3)) ps3.push_back(ps[j]);
    }

    ps1.push_back(node->p1);
    ps1.push_back(node->p2);
    ps1.push_back(node->p);
    ps2.push_back(node->p2);
    ps2.push_back(node->p3);
    ps2.push_back(node->p);
    ps3.push_back(node->p1);
    ps3.push_back(node->p3);
    ps3.push_back(node->p);

    node->n1 = build2(ps1);
    node->n2 = build2(ps2);
    node->n3 = build2(ps3);

    return node;
}

Node* build(vector<Point>& ps)
{
    Node* node = new Node;

    auto v = buildHull(ps);

    if(v.size() == 3)
    {
        node->left = 0, node->right = 0;
        node->nxt = build2(ps);
        return node;
    }

    int j = max(3, (int)ps.size() / 2);

    vector<Point> ps1, ps2;

    for(int i = 0; i < j; i++) ps1.push_back(ps[i]);
    for(int i = j - 1; i < ps.size(); i++) ps2.push_back(ps[i]);
    ps2.push_back(ps[0]);

    node->left = build(ps1);
    node->right = build(ps2);
    node->ps1 = ps1;

    return node;
}

int ask(vector<Point>& ps)
{
    cout << "? " << ps.size() << " ";
    for(auto p : ps) cout << p.i + 1 << " ";
    cout << endl;

    string ans;
    cin >> ans;
    return ans[0] == 'Y';
}

int answer(vector<Point>& ps)
{
    cout << "! " << ps.size() << " ";
    for(auto p : ps) cout << p.i + 1 << " ";
    cout << endl;
}

void solve2(Node2* root)
{
    if(!root->n1)
    {
        vector<Point> v(3);
        v[0] = root->p1;
        v[1] = root->p2;
        v[2] = root->p3;
        answer(v);
        return;
    }

    vector<Point> v1(3), v2(3), v3(3);

    v1[0] = v2[0] = v3[0] = root->p;
    v1[1] = v3[1] = root->p1;
    v1[2] = v2[1] = root->p2;
    v2[2] = v3[2] = root->p3;

    if(ask(v1)) solve2(root->n1);
    else if(ask(v2)) solve2(root->n2);
    else solve2(root->n3);
}

void solve(Node* root)
{
    if(!root->left)
    {
        solve2(root->nxt);
        return;
    }

    if(ask(root->ps1)) solve(root->left);
    else root->right;
}

int main()
{
    #ifdef DEBUG
        freopen("in.in", "r", stdin);
    #endif // DEBUG

    ios_base::sync_with_stdio(0);
    cin.tie(0);

    int n;
    cin >> n;

    for(int i = 0; i < n; i++)
    {
        cin >> arr[i].x >> arr[i].y;
        arr[i].i = i;
    }

    vector<Point> ps(n);
    for(int i = 0; i < n; i++) ps[i] = arr[i];

    Node* root = build(ps);

    int q;
    cin >> q;

    for(int i = 0; i < q; i++)
    {
        solve(root);
    }

    return 0;
}
